local GH = GarrisonHelper
local L = GH.L
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};
-- IDs
GH.alchemyIDs = {
	["alchemyExperiment"] = {37270},
	["buildingID"] = 120,
	["rewards"] = {122453,122451,122454,122452,122455,122456},
}


function GH:isAlchemyEnabled()
	if type(GH.db.profile.alchemysupport)==nil then GH.db.profile.alchemysupport=true end
	return GH.db.profile.alchemysupport
end

function events.QUEST_COMPLETE()
	if GH:isAlchemyEnabled() and GetQuestID()==GH.alchemyIDs["alchemyExperiment"][1] then 
		local o
		if GH.db.profile.alchemyreward then
			for i=1,GetNumQuestChoices() do
				if GH:getIDFromLink(GetQuestItemLink("choice", i))==GH.db.profile.alchemyreward[GH:Player()] then
					o = i
					break
				end
			end
		end
		if o then
			GetQuestReward(o);
		end
	end
end

function events.QUEST_TURNED_IN(questID)
	if GH:isAlchemyEnabled() and GetQuestID()==GH.alchemyIDs["alchemyExperiment"][1] and (not GH.db.profile.alchemyreward or not GH.db.profile.alchemyreward[GH:Player()]) then
		-- reward not defined yet. user had to click
		GH.waitforalchemyreward = true
		GH.alchemyrewardcount = {}
		for i=1,#GH.alchemyIDs["rewards"] do
			GH.alchemyrewardcount[GH.alchemyIDs["rewards"][i]] = GetItemCount(GH.alchemyIDs["rewards"][i], false) 
		end
	end
end

function events.BAG_UPDATE_DELAYED()
	if GH:isAlchemyEnabled() and GH.waitforalchemyreward then
		GH.waitforalchemyreward = false
		if GH.alchemyrewardcount then
			for i=1,#GH.alchemyIDs["rewards"] do
				if GH.alchemyrewardcount[GH.alchemyIDs["rewards"][i]] ~= GetItemCount(GH.alchemyIDs["rewards"][i], false) then
					-- found the chosen quest reward, even if other quest rewards can be found in the bags
					GH:AlchemyPopup(GH.alchemyIDs["rewards"][i])
					break
				end
			end
		end
	end
end

--function GH_AlchemyPopup(id)--/run GH_AlchemyPopup()
--	if not id then id = GH.alchemyIDs["rewards"][4] end
function GH:AlchemyPopup(id)
	  StaticPopupDialogs["Alchemy_Reward"] = StaticPopupDialogs["Alchemy_Reward"] or {
		text = "Should |r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF choose\n\124T%s:24\124t |r|cFFFFFF00%s|r|cFFFFFFFF\nfor\n |r|cFFFFFF00"..GH:Player().."|r|cFFFFFFFF\nnext time automatically?",
		button1 = YES,
		button2 = NO,
		button3 = CANCEL,
		OnAccept = function () --yes
			GH.db.profile.alchemyreward = GH.db.profile.alchemyreward or {}
			GH.db.profile.alchemyreward[GH:Player()] = id
		end,
		OnAlt = function ()-- cancel
		end,
		OnCancel = function() --no
			GH.db.profile.alchemyreward = GH.db.profile.alchemyreward or {}
			GH.db.profile.alchemyreward[GH:Player()] = -1
		end,
		showAlert = 1,
		timeout = 0,
		exclusive = 1,
		hideOnEscape = 1,
		whileDead = 1,
	}
	StaticPopup_Show("Alchemy_Reward", select(10,GetItemInfo(id)), GetItemInfo(id));
end

-- need this function GetUnitName("player",true) is not reliable
function GH:Player() return GetUnitName("player").." - "..GetRealmName() end

function GH:GetAlchemyTooltip()
	if GH:isAlchemyEnabled() and GH:hasBuilding(GH.alchemyIDs["buildingID"],2) then
		local buildingName = select(2,C_Garrison.GetBuildingInfo(GH.alchemyIDs["buildingID"]))
		local questName = GH:QuestTitleFromID(GH.alchemyIDs["alchemyExperiment"][1]) or L["daily quest"]
		local tip = GH.tip
		tip:AddLine(" ")
		tip:AddHeader("|cffF5DA81"..tostring(buildingName))
		tip:AddLine("   "..questName, (IsQuestFlaggedCompleted(GH.alchemyIDs["alchemyExperiment"][1]) and "|cff00ff00"..L["completed"].."|r" or "|cffff0000"..L["not completed"].."|r") );
		tip:AddLine(" ", (GH:hasBuilding(GH.alchemyIDs["buildingID"],2,true) and "|cff00ff00"..L["follower assigned"].."|r" or "|cffff0000"..L["no follower assigned"].."|r\n"));
	end
end

function GH:GetAlchemyOpenTaskCount()
	local res = 0
	if GH:isAlchemyEnabled() and GH:hasBuilding(GH.alchemyIDs["buildingID"],2,true) and not IsQuestFlaggedCompleted(GH.alchemyIDs["alchemyExperiment"][1]) then
		res = 1
	end
	return res
end


function GH_alchemyWelcome()
	if  GH:isAlchemyEnabled() and GH:hasBuilding(GH.alchemyIDs["buildingID"],2,true) then
		if not IsQuestFlaggedCompleted(GH.alchemyIDs["alchemyExperiment"][1]) then
			local questName = GH:QuestTitleFromID(GH.alchemyIDs["alchemyExperiment"][1])
			if questName then
				GH:print(questName.." "..L["not completed today."]);
			end
		end
	end
	if  GH:hasBuilding(GH.alchemyIDs["buildingID"],2) then 
		GH.db.profile.alchemyreward[GH:Player()] = GH.db.profile.alchemyreward[GH:Player()] or nil
	end
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) events[event](...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end


--[[
	Needed, because if cache is empty, this information have to be retrieved from server.
]]
function GH:PreLoadAlchemyInfos()
	IsQuestFlaggedCompleted(GH.alchemyIDs["alchemyExperiment"][1])
	C_Garrison.GetBuildingInfo(GH.alchemyIDs["buildingID"])
	GetUnitName("player")
	GetRealmName()
	for i=1,#GH.alchemyIDs["rewards"] do
		GetItemInfo(GH.alchemyIDs["rewards"][i])
	end
end
GH:PreLoadAlchemyInfos() -- TODO move this to an event like addon loaded